/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifdef TEST
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define Int8 int
#else
#include <PalmOS.h>
#include "MathLib.h"
#include "geoidalheight.h"
#include "util.h"
#endif
#include "geoidalheightparam.h"

Int8 GetGeoidalHeightRad(double lat, double lon)
{
	return(GetGeoidalHeightDeg(180.*lat/M_PI, 180.*lon/M_PI));
}

Int8 GetGeoidalHeightDeg(double lat, double lon)
{
	Int8 ilat, ilon, h1, h2, h3, h4;
	double h, hon1, hon2;
	double cat, con;

	lat  = lat+90;
	lon  = lon+180;
	ilat = (Int8)(lat/10.);
	ilon = (Int8)(lon/10.);
	h1   = GEOH[ilat  ][ilon  ];
	h2   = GEOH[ilat  ][ilon+1];
	h3   = GEOH[ilat+1][ilon  ];
	h4   = GEOH[ilat+1][ilon+1];
	con  = fmod(lon, 10)/10;
	cat  = fmod(lat, 10)/10;
	hon1 = h1+(h2-h1)*con;
	hon2 = h3+(h4-h3)*con;
	h    = hon1+(hon2-hon1)*cat;

#ifdef TEST
	printf("corners: %d:%d:%d:%d\n", h1, h2, h3, h4);
	printf("con: %f\n", con);
	printf("cat: %f\n", cat);
	printf("hon1: %f hon2: %f\n", hon1, hon2);
	printf("h: %f\n", h);
#endif

	if(h < 0)
		h -= .5;
	else
		h += .5;

	return((Int8)h);
}

#ifdef TEST
int main(int argc, char **argv)
{
	double lat, lon;
	Int8 height;

	if(argc != 3)
	{
		fprintf(stderr, "usage: %s <lat> <lon>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	lat = atof(argv[1]);
	lon = atof(argv[2]);

	printf("lat/lon: %f/%f\n", lat, lon);

	height = GetGeoidalHeight(lat, lon);

	printf("height: %d", height);
}
#endif

