/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include <handera/Vga.h>
#include "util.h"
#include "resource.h"
#include "help.h"

static void HelpSetID(UInt16 msgId);

static Boolean is_error;
static MemHandle msgh	= NULL;
static Char *msg		= NULL;
static Char *title_help	= NULL;
static Char *title_err	= NULL;

void HelpInit()
{
	title_help = UtilGetResourceString(HelpTITLEHELP);
	title_err  = UtilGetResourceString(HelpTITLEERR);
	is_error = false;
}

void HelpFinish()
{
	HelpReleaseString();
	if(title_help != NULL)
	{
		UtilFree(title_help);
		title_help = NULL;
	}
	if(title_err != NULL)
	{
		UtilFree(title_err);
		title_err = NULL;
	}
}

const Char *HelpGetString()
{
	return(msg);
}

void HelpReleaseString()
{
	if(msg != NULL)
		msg = NULL;
	if(msgh != NULL)
	{
		MemHandleUnlock(msgh);
		UtilReleaseResourceStringHandle(msgh);
		msgh = NULL;
	}
}

void HelpSetHelpID(UInt16 helpMsgId)
{
	is_error = false;
	HelpSetID(helpMsgId);
}

void HelpSetErrorID(UInt16 errMsgId)
{
	is_error = true;
	HelpSetID(errMsgId);
}

static void HelpSetID(UInt16 msgId)
{
	HelpReleaseString();
	msgh = UtilGetResourceStringHandle(msgId);
	msg  = MemHandleLock(msgh);
}

void HelpSetErrorMsg(Char *helpmsg)
{
	is_error = true;
	HelpReleaseString();
	msg = helpmsg;
}

Boolean HelpIsError()
{
	return(is_error);
}

const Char *HelpGetTitleHelp()
{
	return(title_help);
}

const Char *HelpGetTitleErr()
{
	return(title_err);
}

