/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include "log.h"
#include "main.h"
#include "util.h"
#include "nmea.h"
#include "resource.h"

static Boolean is_showlog = false;
static Boolean is_logging = false;
static DmOpenRef log_db;
static UInt16 count, index;


void LogStart()
{
	LocalID db_id;

	if(is_logging == true)
		return;

	if(is_showlog == true)
	{
		UtilError(LogUSINGLOG);
		return;
	}

	db_id  = DmFindDatabase(0, CGDBLog);
	if(db_id == 0)
	{
		if(DmCreateDatabase(0, CGDBLog, CGCreator, CGDBLogType, false) != errNone)
		{
			UtilError(LogDBCREATEFAILED);
			return;
		}
		db_id  = DmFindDatabase(0, CGDBLog);
	}

	log_db = DmOpenDatabase(0, db_id, dmModeWrite);
	if(log_db == 0)
	{
		UtilError(LogDBOPENFAILED);
		return;
	}

	is_logging = true;
}

void LogEnd()
{
	LocalID db_id;

	if(is_logging == false)
		return;

	DmCloseDatabase(log_db);

	db_id = DmFindDatabase(0, CGDBLog);
	if(db_id != 0)
	{
		UInt16 attr;
		DmDatabaseInfo(0, db_id, NULL, &attr, NULL, NULL, NULL,
						NULL, NULL, NULL, NULL, NULL, NULL);
		attr |= dmHdrAttrBackup;
		DmSetDatabaseInfo(0, db_id, NULL, &attr, NULL, NULL, NULL,
						NULL, NULL, NULL, NULL, NULL, NULL);
	}

	is_logging = false;
}

Boolean LogIsLogging()
{
	return(is_logging);
}

void LogAppend(char *data)
{
	MemHandle record_h;
	MemPtr record;
	UInt16 rec_index, len;

	if(is_logging == false)
		return;

	len = StrLen(data) + 1;
	rec_index = dmMaxRecordIndex;
	record_h = DmNewRecord(log_db, &rec_index, len);
	if(record_h == 0)
	{
		UtilError(LogRECCREATEFAILED);
		return;
	}
	record = MemHandleLock(record_h);
	DmStrCopy(record, 0, data);
	MemHandleUnlock(record_h);
	DmReleaseRecord(log_db, rec_index, true);
}

void LogRemove()
{
	LocalID db_id;

	LogEnd();
	db_id = DmFindDatabase(0, CGDBLog);
	if(db_id != 0)
		DmDeleteDatabase(0, db_id);
}

void LogOpen()
{
	LocalID db_id;

	if(is_showlog == true)
		return;

	if(is_logging == true)
	{
		UtilError(LogWRITINGLOG);
		return;
	}

	db_id = DmFindDatabase(0, CGDBLog);
	if(db_id == 0)
	{
		UtilError(LogNOLOG);
		return;
	}

	log_db = DmOpenDatabase(0, db_id, dmModeReadOnly);
	if(log_db == 0)
	{
		UtilError(LogDBOPENFAILED);
		return;
	}
	count = DmNumRecords(log_db);
	index = 0;

	is_showlog = true;
}

void LogClose()
{
	if(is_showlog == false)
		return;

	DmCloseDatabase(log_db);

	is_showlog = false;
}

Boolean LogIsShowLog()
{
	return(is_showlog);
}

Boolean LogNextLine(char *buffer)
{
	MemHandle handle;
	MemPtr data;

	if((is_showlog == false) || (count == 0))
		return(false);

	index = (index+1) % count;
	handle = DmGetRecord(log_db, index);
	if(handle == NULL)
	{
		UtilError(LogRECREADFAILED);
		return(false);
	}
	data = MemHandleLock(handle);
	StrCopy(buffer, data);
	MemHandleUnlock(handle);
	DmReleaseRecord(log_db, index, false);
	return(true);
}

