/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifdef MOVINGMAP
#include <PalmOS.h>
#include <handera/Vga.h>
#include "JpegLibPalm.h"
//#include "pnoJpeg.h"
#include "track.h"
#include "draw.h"
#include "main.h"
#include "util.h"
#include "nmea.h"
#include "forms.h"
#include "resource.h"
//#include "pdouble.h"
#include "def.h"

typedef struct
{
	Int16 fileversion;
	Int16 width;
	Int16 height;
	Int16 lux;
	Int16 luy;
	double lulat;
	double lulon;
	Int16 rlx;
	Int16 rly;
	double rllat;
	double rllon;
} CoorMap;

static void GPSHandler(FormPtr form, nmea_type *nmea) CODE4_SECTION;
static Boolean SizeForm(FormPtr form) CODE4_SECTION;
static void HandlePen(FormPtr form, eventsEnum type, Int16 ex, Int16 ey) CODE4_SECTION;
static void DrawMap(FormPtr form) CODE4_SECTION;
static void CalcMapping(CoorMap *mapping) CODE4_SECTION;
//static Char *GetCoor(double pos, Latlon latlon) CODE4_SECTION;

static Char name[MAX_MAP_NAME_LENGTH];
static Boolean form_ready = false;
static RectangleType formrect;
static BitmapType *bitmap;
static Boolean pendown, jpegproblem;
static double lat1, lat2, lon1, lon2;
static double flat, flon;
static Int16 x, y, px, py;

Boolean MapDrawFormHandler(EventPtr event)
{
	Boolean handled = false;
	FormPtr	form;

	form = FrmGetActiveForm();
	switch(event->eType)
	{
		case frmOpenEvent:
			form_ready = false;
			bitmap = NULL;
			pendown = false;
			jpegproblem = false;
			x = y = px = py = 0;
			lat1 = lat2 = lon1 = lon2 = 0.;
			if(UtilIsVGA())
				VgaFormModify(form, vgaFormModify160To240);
			SizeForm(form);
			FrmDrawForm(form);
			DrawMap(form);
			form_ready = true;
			handled = true;
			break;
		case frmCloseEvent:
			if(bitmap != NULL)
			{
				BmpDelete(bitmap);
				bitmap = NULL;
			}
			form_ready = false;
			break;
		case frmUpdateEvent:
			DrawMap(form);
			handled = true;
			break;
		case winDisplayChangedEvent:
		case displayExtentChangedEvent:
			if((form_ready == true) && SizeForm(form))
				DrawMap(form);
			handled = true;
			break;
		case penDownEvent:
			HandlePen(form, event->eType, event->screenX, event->screenY);
			break;
		case penUpEvent:
			HandlePen(form, event->eType, event->screenX, event->screenY);
			break;
		case penMoveEvent:
			HandlePen(form, event->eType, event->screenX, event->screenY);
			break;
		case GpsNmeaEvent:
			GPSHandler(form, CASTNMEA(event));
			break;
		default:
			break;
	}
	return(handled);
}

static void HandlePen(FormPtr form, eventsEnum type, Int16 ex, Int16 ey)
{
	if(jpegproblem == true)
		return;

	switch(type)
	{
		case penDownEvent:
			pendown = true;
			x = ex;
			y = ey;
		break;
		case penUpEvent:
			pendown = false;
			x = 0;
			y = 0;
		break;
		case penMoveEvent:
			px += ex - x;
			py += ey - y;
			x = ex;
			y = ey;
			DrawMap(form);
		break;
		default:
			/* nothing */
		break;
	}
}

static Boolean SizeForm(FormPtr form)
{
	RectangleType displayBounds;
	Coord diff, diff2;

	UtilGetFrmBounds(form, &formrect);
	UtilGetDpyBounds(&displayBounds);

	diff = displayBounds.extent.y - (formrect.topLeft.y + formrect.extent.y);
	diff2 = diff >> 1;

	if(diff)
	{
		formrect.extent.y += diff;
		WinSetBounds(FrmGetWindowHandle(form), &formrect);
		return(true);
	}
	return(false);
}

static void GPSHandler(FormPtr form, nmea_type *nmea)
{
	if(jpegproblem == true)
		return;

	if(nmea->is_new_gga || nmea->is_new_nexrmc)
	{
		Int16 x1, y1, x2, y2;
		Int16 sx, sy, sw, sh;

		DrawMap(form);

		sx = formrect.topLeft.x;
		sy = formrect.topLeft.y;
		sw = formrect.extent.x;
		sh = formrect.extent.y;

/*
TRACE("lat : %s -> %s", pdouble(nmea->lat), GetCoor(nmea->lat, LATLON_LAT));
TRACE("lon : %s -> %s", pdouble(nmea->lon), GetCoor(nmea->lon, LATLON_LON));
TRACE("lat1: %s -> %s", pdouble(lat1), GetCoor(lat1, LATLON_LAT));
TRACE("lat2: %s -> %s", pdouble(lat2), GetCoor(lat2, LATLON_LAT));
TRACE("lon1: %s -> %s", pdouble(lon1), GetCoor(lon1, LATLON_LON));
TRACE("lon2: %s -> %s", pdouble(lon2), GetCoor(lon2, LATLON_LON));
TRACE("flat: %s", pdouble(flat));
TRACE("flon: %s", pdouble(flon));
TRACE("px  : %d", px);
TRACE("py  : %d", py);
*/

		x1 = sx;
		x2 = x1+sw;
		y1 = (nmea->lat-lat1)/flat;
		y2 = y1;
/*
TRACE("LAT x1  : %d", x1);
TRACE("LAT x2  : %d", x2);
TRACE("LAT y1  : %d", y1);
TRACE("LAT y2  : %d", y2);
*/
		WinDrawGrayLine(x1, y1+py, x2, y2+py);

		x1 = (nmea->lon-lon1)/flon;
		x2 = x1;
		y1 = sy;
		y2 = y1+sh;
/*
TRACE("LON x1  : %d", x1);
TRACE("LON x2  : %d", x2);
TRACE("LON y1  : %d", y1);
TRACE("LON y2  : %d", y2);
*/
		WinDrawGrayLine(x1+px, y1, x2+px, y2);

		nmea->is_new_gga = false;
	}
}

static void DrawMap(FormPtr form)
{
	Boolean loaded = false;
	Err err;

	if(jpegproblem == true)
		return;

	if(bitmap == NULL)
	{
		FileHand fh;
		JpegLibData data;
		JpegLibDecodeOptions dopt;
		UInt16 jpegLibRef, w, h, nc, usecount;

		StrCopy(name, "cotogpsmap.jpg");
		err = SysLibFind(jpegLibName, &jpegLibRef);
		if(err != errNone)
		{
			err = SysLibLoad(sysResTLibrary, jpegLibCreator, &jpegLibRef);
			loaded = true;
		}
		if(err == errNone)
		{
			err = JpegLibOpen(jpegLibRef, jpegLibCurrentVersion);
		}
		if(err != errNone)
		{
			jpegproblem = true;
			if(loaded)
				SysLibRemove(jpegLibRef);
			UtilAlert(MapJPEGLIBOPENFAILED);
			return;
		}
		else
		{
			fh = FileOpen(0, name, 0, 0, fileModeReadOnly, &err);
			if(err != errNone)
			{
				jpegproblem = true;
				UtilAlert(MapJPEGFILEOPENFAILED);
			}
			else
			{
				CoorMap mapping;

				FileRead(fh, &mapping, sizeof(CoorMap), 1, &err);

				// skip to next block (yes, we are wasting almost 4KB)
				FileSeek(fh, 4096, fileOriginBeginning);

				JpegLibCreateDataFileStreaming(jpegLibRef, &data, fh);
				JpegLibCreateDecOptions(jpegLibRef, &dopt);
				JpegLibSetDecOptDestBitmap(jpegLibRef, &dopt);
				//JpegLibSetDecOptScale(jpegLibRef, &dopt, 2);
				err = JpegLibRead(jpegLibRef, &data, &dopt, &w, &h, &nc, (void **)&bitmap);
				FileClose(fh);

				if(err != errNone)
				{
					jpegproblem = true;
					UtilAlert(MapJPEGFILEREADFAILED);
				}

				CalcMapping(&mapping);
			}
		}


		err = JpegLibClose(jpegLibRef, &usecount);
		if(err == errNone && usecount == 0)
			SysLibRemove(jpegLibRef);
	}

	if(bitmap != NULL)
	{
		Int16 slen, fw, posx, posy;

		slen = StrLen(name);
		fw = FntCharsWidth(name, slen);
		posx = (formrect.extent.x - fw)/2;
		posy = formrect.extent.y - FntCharHeight() - 2;

		WinEraseRectangle(&formrect, 0);
		WinDrawBitmap(bitmap, px, py);
		if(UtilIsOS(3, 5))
		{
			WinDrawOperation odo = WinSetDrawMode(winInvert);
			WinPaintChars(name, slen, posx, posy);
			WinSetDrawMode(odo);
		}
		else
			WinDrawChars(name, slen, posx, posy);
	}
}

static void CalcMapping(CoorMap *m)
{
/*
TRACE("ver:   %d", m->fileversion);
TRACE("w  :   %d", m->width);
TRACE("h  :   %d", m->height);
TRACE("lux:   %d", m->lux);
TRACE("luy:   %d", m->luy);
TRACE("lulat: %s", GetCoor(m->lulat, LATLON_LAT));
TRACE("lulon: %s", GetCoor(m->lulon, LATLON_LON));
TRACE("rlx:   %d", m->rlx);
TRACE("rly:   %d", m->rly);
TRACE("rllat: %s", GetCoor(m->rllat, LATLON_LAT));
TRACE("rllon: %s", GetCoor(m->rllon, LATLON_LON));
*/

	flat = (m->rllat - m->lulat) / (m->rly - m->luy);
	lat1 = m->lulat - flat*m->luy;
	lat2 = m->rllat + flat*(m->height-m->rly);

	flon = (m->lulon - m->rllon) / (m->lux - m->rlx);
	lon1 = m->lulon - flon*m->lux;
	lon2 = m->rllon + flon*(m->width-m->rlx);
/*
TRACE("lat1: %s -> %s", pdouble(lat1), GetCoor(lat1, LATLON_LAT));
TRACE("lat2: %s -> %s", pdouble(lat2), GetCoor(lat2, LATLON_LAT));
TRACE("lon1: %s -> %s", pdouble(lon1), GetCoor(lon1, LATLON_LON));
TRACE("lon2: %s -> %s", pdouble(lon2), GetCoor(lon2, LATLON_LON));
*/
}

/*
static Char *GetCoor(double pos, Latlon latlon)
{
	static char buf[MAX_LABEL_LENGTH];
	Int32 deg, min, frac;
	Char c;

	NMEAGetDeg_DMM(pos, 3, &deg, &min, &frac, &c, latlon);
	StrPrintF(buf, "%c %ld %ld.%03ld'", c, deg, min, frac);
	return(buf);
}
*/

#endif /* MOVINGMAP */
