/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _marker_h_
#define _marker_h_

#include "main.h"
#include "resource.h"
#include "def.h"

typedef struct
{
	double lon, lat;
	Char name[MAX_MARKER_NAME_LENGTH];
} marker_type;

typedef struct
{
	DmOpenRef db;
	UInt16 index, tindex, offset, size, mode;
	MemHandle mhandle;
} marker_note_type;

void MarkerInit() CODE2_SECTION;
void MarkerFinish() CODE2_SECTION;
void MarkerSetDestination(UInt16 dest) CODE2_SECTION;
void MarkerClearDestination() CODE2_SECTION;
Boolean MarkerHasDestination() CODE2_SECTION;
Int32 MarkerGetCourse(nmea_type *nmea) CODE2_SECTION;
UInt32 MarkerGetDistance(nmea_type *nmea) CODE2_SECTION;

void MarkerEdit(Int16 index, Boolean is_editable) CODE2_SECTION;
Int16 MarkerGetEditIndex() CODE2_SECTION;
void MarkerSetViewIndex(Int16) CODE2_SECTION;
Int16 MarkerGetViewIndex() CODE2_SECTION;
Boolean MarkerIsEditable() CODE2_SECTION;

DmOpenRef MarkerOpenDB(UInt16 mode) CODE2_SECTION;
void MarkerCloseDB(DmOpenRef db) CODE2_SECTION;
Boolean MarkerMoveMarker(UInt16 from, UInt16 to) CODE2_SECTION;
Boolean MarkerSort(DmComparF sort) CODE2_SECTION;
Boolean MarkerQSort() CODE2_SECTION;
Boolean MarkerDSort(double lon, double lat) CODE2_SECTION;
UInt16 MarkerAddMarker(Char *name, double lon, double lat, Err *err) CODE2_SECTION;
void MarkerUpdateMarker(UInt16 index, Char *name,
						double lon, double lat, Err *err) CODE2_SECTION;
void MarkerDelMarker(UInt16 index, Err *err) CODE2_SECTION;
UInt32 MarkerGetMarkerCount() CODE2_SECTION;
UInt32 MarkerGetMarkerCountInCategory(UInt16 category) CODE2_SECTION;
Boolean MarkerGetMarker(UInt16 index, marker_type *marker) CODE2_SECTION;
Boolean MarkerGetMarkerDescription(marker_note_type *note) CODE2_SECTION;
void MarkerUpdateMarkerDescription(marker_note_type *note, Boolean update) CODE2_SECTION;
void MarkerBeam(Int16 index) CODE2_SECTION;
void MarkerBeamList() CODE2_SECTION;
void MarkerBeamDB() CODE2_SECTION;
void MarkerToClipboard(Int16 index) CODE2_SECTION;

#endif /* _marker_h_ */
