/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _meter_h_
#define _meter_h_

#include "nmea.h"
#include "def.h"

#define VARIO_MAX_METER 10
#define VARIO_MUL (180/VARIO_MAX_METER)

typedef enum
{
	TYPE_COMPASS, TYPE_ODO, TYPE_VARIO
} MeterType;

struct meter_type_struct;
typedef struct meter_type_struct meter_type;

struct draw_area_struct
{
	/* center for painting operations */
	Coord center_x, center_y;
	/* where we paint */
	RectangleType rect;
};
typedef struct draw_area_struct draw_area_type;

struct meter_type_struct
{
	/* wtf am I */
	MeterType type;
	/* if double buffering, this is only relation to screen coords! */
	Coord x_dbuf, y_dbuf;
	/* radius and internal radius for needle */
	Coord radius, iradius;
	/* needle corners */
	Coord cx[5], cy[5];

	Int32 aval;
	Int32 aval2;
	UInt32 dval;
	UInt32 fixtime;	/* 1/100th of a sec */
	Boolean has_aval;
	Boolean has_aval2;
	Boolean has_dval;
	Boolean wants_dval;
	Boolean is_fixed;
	Boolean do_dbuf;
	WinHandle dbuf;
	FontID font, ufont;
	Boolean firsttime;
	Boolean eraselast;
	Int16 square;

	/* these are the drawing rectangles */
	draw_area_type area_screen;
	draw_area_type area_virt;

	/* a flag for moveable/non moveable */
	Boolean mv_back;
	Boolean mv_scale;
	Boolean mv_needle;

	/* drawing function pointers */
	void (*draw_back)   (meter_type *meter, draw_area_type *area);
	void (*draw_scale)  (meter_type *meter, draw_area_type *area);
	void (*draw_needle) (meter_type *meter, draw_area_type *area);
	void (*draw_value)  (meter_type *meter, draw_area_type *area);

	void (*meter_update)(meter_type *meter, nmea_type *nmea);
};

void MeterInit(meter_type *meter, MeterType type) CODE3_SECTION;
void MeterFinish(meter_type *meter) CODE3_SECTION;
void MeterSetFixed(meter_type *meter, Boolean b) CODE3_SECTION;
void MeterSetDoublebuffered(meter_type *meter, Boolean b) CODE3_SECTION;
void MeterSetSize(meter_type *meter, Coord x, Coord y, Coord w, Coord h) CODE3_SECTION;
void MeterDraw(meter_type *meter) CODE3_SECTION;
void MeterErase(meter_type *meter) CODE3_SECTION;
void MeterUpdate(meter_type *meter, nmea_type *nmea) CODE3_SECTION;
Boolean MeterPenInside(meter_type *meter, EventPtr event) CODE3_SECTION;

#endif /* _meter_h_ */
