/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>

#define UtilCharIsDigit(c) (((c) >= '0') && ((c) <= '9'))
#define UtilCharIsAlpha(c) ((((c) >= 'a') && ((c) <= 'z')) || (((c) >= 'A') && ((c) <= 'Z')))
#define UtilCharIsUpper(c) (((c) >= 'A') && ((c) <= 'Z'))

static unsigned int NMEAHex2Int(char *buf)
{
	unsigned int n1, n2;
	char h = buf[0];
	char l = buf[1];

	n1 = n2 = 0;

	if(UtilCharIsDigit(h))
		n1 = h - '0';
	else
		if(UtilCharIsAlpha(h))
			n1 = h - (UtilCharIsUpper(h) ? 'A' - 10 : 'a' - 10);

	if(UtilCharIsDigit(l))
		n2 = l - '0';
	else
		if(UtilCharIsAlpha(l))
			n2 = l - (UtilCharIsUpper(l) ? 'A' - 10 : 'a' - 10);

	return((n1 << 4)+n2);
}

int main(int argc, char **argv)
{
	char *cp;
	char *nmeabuf;
	unsigned int cs_is, cs_should;

	if(argc != 2)
	{
		fprintf(stderr, "usage: %s <nmea-sentence>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	nmeabuf = argv[1];
	cp = strchr(nmeabuf, '*');
	if(cp)
	{
		*cp++ = '\0';	// cut off checksum
		cs_should = NMEAHex2Int(cp);

		for(cs_is = 0, cp = nmeabuf+1; *cp; cp++)
			cs_is ^= *cp;

		fprintf(stdout, "CS. is: 0x%x, should: 0x%x\n", cs_is, cs_should);
	}

	return(EXIT_SUCCESS);
}
