/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _prefs_h_
#define _prefs_h_

#include <PalmOS.h>
#include <BtLib.h>
#include "marker.h"
#include "util.h"
#include "main.h"
#include "def.h"

typedef enum
{
	METRIC = 0,
	NAUTICAL,
	STATUTE,
	GOLF
} Unit;

typedef struct
{
	UInt16 start_form;
	Int16 time_offset;
	Int16 port_index;
	Int16 port_speed_index;
	Int16 format_distspeed;
	Int16 format_alt;
	Int16 format_coord;
	Boolean useNMEAAltWGS84;
	Boolean correctWGS84;
	Int8  drift;
	UInt16 maxspeed;

	Int16 track_interval_index;
	Boolean track_side_inside;
	Int16 track_offset_m;
	Int16 track_offset_c;
	UInt16 track_offset_c_l;
	Int16 track_beep;
	Int16 track_format_dist;
	Int16 track_format_area;
	double track_last_dist;
	double track_last_area;
	Boolean track_has_last_dist;
	Boolean track_has_last_area;
	Boolean sat_data_doastro;

	double dist_f;
	Int32 max_speed;
	Int32 min_alt;
	Int32 max_alt;
	Boolean show_rawdata; // unused since V1.3.3
	Boolean compass_fixed;
	UInt8 help_font;

	UInt16 marker_idx;
	UInt16 marker_category;
	UInt16 track_category;
	UInt16 map_category;
	Char track_name[MAX_TRACK_NAME_LENGTH];
	Char map_name[MAX_MAP_NAME_LENGTH];
	BtLibDeviceAddressType btaddress;
} CAppPrefs;

void PrefsInit() CODE3_SECTION;
void PrefsFinish() CODE3_SECTION;
void PrefsSave() CODE3_SECTION;
CAppPrefs *PrefsGetPrefs() CODE3_SECTION;
Boolean PrefsHasPrefs() CODE3_SECTION;
void PrefsSetPrefs(Boolean b) CODE3_SECTION;
UInt16 PrefsGetDec() CODE3_SECTION;

void PrefsAltToPref(Unit from, Int32 *val, Char **unit) CODE3_SECTION;
void PrefsNauticToPref(double *val, Char **unit) CODE3_SECTION;
void PrefsSpeedToPref(double *val, Char **unit) CODE3_SECTION;
void PrefsUpdateLabelOffset(FormPtr form, UInt16 id) CODE3_SECTION;
double PrefsOffsetToNM(double offset) CODE3_SECTION;
void PrefsUpdateLabelLatLon(FormPtr form,
							UInt16 lat_id, UInt16 lon_id,
							Boolean is_short) CODE3_SECTION;
void PrefsUpdateLabelCoord(FormPtr form,
							UInt16 lat_id, UInt16 lon_id,
							double lat_val, double lon_val) CODE3_SECTION;

void PrefsUpdateLabelAlt(FormPtr form, UInt16 id,
							Int32 alt, char c,
							Int32 alt84, char c84,
							double lat, double lon) CODE3_SECTION;
void PrefsUpdateLabelNautic(FormPtr form, UInt16 id, double value) CODE3_SECTION;
void PrefsUpdateLabelSpeed(FormPtr form, UInt16 id, double value) CODE3_SECTION;
const Char *PrefsTrackDistUnit() CODE3_SECTION;
const Char *PrefsTrackAreaUnit() CODE3_SECTION;
double PrefsTrackDistToPref(double val) CODE3_SECTION;
double PrefsTrackAreaToPref(double val) CODE3_SECTION;
Err PrefsGetBTDevice() CODE3_SECTION;

#endif /* _prefs_h_ */
