/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include <handera/Vga.h>
#include "main.h"
#include "forms.h"
#include "util.h"
#include "serial.h"
#include "time.h"
#include "prefs.h"
#include "resource.h"
#include "def.h"

Boolean ConfigFormHandler(EventPtr event)
{
	FormPtr	form;
	Int16 index, cnt;
	Char buf[5];
	Char **items;
	CAppPrefs *prefs;
	Boolean handled = false;

	form = FrmGetActiveForm();
	prefs = PrefsGetPrefs();

	switch(event->eType)
	{
		case frmOpenEvent:
				if(UtilIsVGA())
					VgaFormModify(form, vgaFormModify160To240);

				FrmDrawForm(form);

				if(UtilHasBT() == false)
					UtilShowObject(form, ConfigBTButtonID, false);
				SerialGetSpeeds(&cnt, &items);
				UtilSetList(form, BaudListID, BaudPopupTriggerID,
							items, cnt, prefs->port_speed_index);

				SerialGetDevices(&cnt, &items);
				if(prefs->port_index >= cnt)
					prefs->port_index = SerialGetDefPort();
				UtilSetList(form, PortListID, PortPopupTriggerID,
							items, cnt, prefs->port_index);

				TimeGetOffsets(&cnt, &items);
				UtilSetList(form, TimeOffsetListID, TimeOffsetTriggerID,
							items, cnt, prefs->time_offset);
				//if(UtilIsOS(4, 0))
				//	UtilSetEnabled(form, TimeOffsetTriggerID, false);

				UtilSetListSelection(form, DistSpeedListID,
							DistSpeedTriggerID, prefs->format_distspeed);

				UtilSetListSelection(form, AltFormatListID,
							AltFormatTriggerID, prefs->format_alt);

				UtilSetListSelection(form, CoordFormatListID,
							CoordFormatTriggerID, prefs->format_coord);

				UtilSetControlValue(form, UseNMEAWGS84ID, prefs->useNMEAAltWGS84);
				UtilSetControlValue(form, CorrectWGS84ID, prefs->correctWGS84);
				StrPrintF(buf, "%d", prefs->drift);
				UtilSetFieldText(form, DriftID, buf);
				StrPrintF(buf, "%d", prefs->maxspeed);
				UtilSetFieldText(form, MaxSpeedID, buf);
				handled = true;
			break;
		case frmCloseEvent:
			break;
		case ctlSelectEvent:
				if(event->data.ctlSelect.controlID == ConfigBTButtonID)
				{
					Err err = PrefsGetBTDevice();
					if(err == errNone)
					{
						// TODO fill remote device's data into GUI
					}
				}
				if(event->data.ctlSelect.controlID == ConfigOKButtonID)
				{
					ListType *list;
					Boolean restart = false;

					list = UtilGetObjectPtr(form, BaudListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->port_speed_index != index)
						{
							prefs->port_speed_index = index;
							restart = true;
						}
					}

					list = UtilGetObjectPtr(form, PortListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->port_index != index)
						{
							prefs->port_index = index;
							restart = true;
						}
					}

					list = UtilGetObjectPtr(form, TimeOffsetListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->time_offset != index)
						{
							prefs->time_offset = index;
							TimeSetCurrentOffset(index);
						}
					}

					list = UtilGetObjectPtr(form, DistSpeedListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->format_distspeed != index)
						{
							prefs->format_distspeed = index;
						}
					}

					list = UtilGetObjectPtr(form, AltFormatListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->format_alt != index)
						{
							prefs->format_alt = index;
						}
					}

					list = UtilGetObjectPtr(form, CoordFormatListID);
					index = LstGetSelection(list);
					if(index != noListSelection)
					{
						if(prefs->format_coord != index)
						{
							prefs->format_coord = index;
						}
					}

					prefs->useNMEAAltWGS84 = UtilGetControlValue(form, UseNMEAWGS84ID);
					prefs->correctWGS84    = UtilGetControlValue(form, CorrectWGS84ID);
					prefs->drift           = StrAToI(UtilGetString(form, DriftID));
					prefs->maxspeed        = StrAToI(UtilGetString(form, MaxSpeedID));

					if((PrefsHasPrefs() == false) || (restart == true))
					{
						PrefsSetPrefs(true);
						PrefsSave();
						MainStopGPS();
						MainStartGPS();
					}
					else
					{
						if(SerialIsRunning() == false)
							MainStartGPS();
					}
					FrmUpdateForm(UtilLastFormId(), frmRedrawUpdateCode);
					FrmReturnToForm(0);

					handled = true;
				}
			break;
		default:
			break;
	}
	return(handled);
}

