/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include <handera/Vga.h>
#include "main.h"
#include "util.h"
#include "nmea.h"
#include "time.h"
#include "draw.h"
#include "resource.h"
#include "prefs.h"
#include "forms.h"
#include "def.h"

static void RawHandler(FormPtr form, char *data) CODE2_SECTION;
static void GPSHandler(FormPtr form, nmea_type *nmea) CODE2_SECTION;
static void SizeForm(FormPtr form) CODE2_SECTION;

static UInt16 title_height;
static UInt16 maxwidth, maxheight;
static Boolean wasdown, toggle;
static Coord drawy;
static IndexedColorType cred, cblue;

Boolean RawDataFormHandler(EventPtr event)
{
	RectangleType rect;
	Boolean handled = false;
	FormPtr	form = FrmGetActiveForm();

	switch(event->eType)
	{
		case frmOpenEvent:
			wasdown = false;
			title_height = 16;
			toggle = false;
			if(UtilIsVGA())
			{
				title_height = VgaGetFrmTitleHeight()+1;
				VgaFormModify(form, vgaFormModify160To240);
			}
			drawy = title_height;
			if(UtilIsResizeableNoDia())
				SizeForm(form);
			FrmDrawForm(form);
			UtilGetFrmBounds(form, &rect);
			maxwidth  = rect.extent.x;
			maxheight = rect.extent.y;
			if(UtilHasColor())
			{
				RGBColorType ctype;
				ctype.r = 255;
				ctype.g = 0;
				ctype.b = 0;
				cred = WinRGBToIndex(&ctype);
				ctype.r = 0;
				ctype.g = 0;
				ctype.b = 255;
				cblue = WinRGBToIndex(&ctype);
			}
			handled = true;
			break;
		case penDownEvent:
			if(event->screenY > title_height)
				wasdown = true;
			break;
		case penUpEvent:
			if(wasdown)
				UtilReturnToLastForm();
			wasdown = false;
			break;
		case winDisplayChangedEvent:
		case displayExtentChangedEvent:
			SizeForm(form);
			FrmDrawForm(form);
			handled = true;
			break;
		case GpsRawEvent:
				RawHandler(form, CASTRAW(event));
				handled = true;
			break;
		case GpsNmeaEvent:
				GPSHandler(form, CASTNMEA(event));
			break;
		default:
			break;
	}
	return(handled);
}

static void RawHandler(FormPtr form, char *data)
{
	RectangleType rect;
	Int16 fh, slen;
	UInt16 len;
	IndexedColorType ocolor = 0;

	if((data == NULL) || (wasdown == true))
		return;

	slen = StrLen(data);
	fh = FntCharHeight();
	if(UtilHasColor())
	{
		if(toggle)
			ocolor = WinSetTextColor(cred);
		else
			ocolor = WinSetTextColor(cblue);
	}

	while(slen > 0)
	{
		len = FntWordWrap(data, maxwidth);

		RctSetRectangle(&rect, 0, drawy, maxwidth, fh);
		WinEraseRectangle(&rect, 0);
		WinDrawChars(data, len, 0, drawy);

		data += len;
		slen -= len;
		drawy += fh;
		if(drawy > (maxheight-fh))
		{
			drawy = title_height;
			toggle = !toggle;
		}
	}
	if(UtilHasColor())
		WinSetTextColor(ocolor);
}

static void GPSHandler(FormPtr form, nmea_type *nmea)
{
	MainSetGPSTitle(form);
}

static void SizeForm(FormPtr form)
{
	RectangleType curBounds, displayBounds;
	Coord diff;

	UtilGetFrmBounds(form, &curBounds);
	UtilGetDpyBounds(&displayBounds);

	diff  = displayBounds.extent.y - (curBounds.topLeft.y + curBounds.extent.y);

	if(diff)
	{
		curBounds.extent.y += diff;
		WinSetBounds(FrmGetWindowHandle(form), &curBounds);
		maxheight += diff;
	}
}

