
/*
 * This code is (c) "Hawkins, Mark" <mark.hawkins@ngc.com>
 * of Soaring Pilot fame, thank you!
 */

#include <PalmOS.h>  // all the system toolbox headers
#include "soarGEOMAG.h"
#include "soarMAGCOFF.h"
#include "MathLib.h"
#include "util.h"

#define MAXDEG 12

static float CalcDeviationD(float alt, float glat, float glon, float time) CODE4_SECTION;

/*************************************************************************/

double CalcDeviation(double alt, double glat, double glon, double time)
{
	return((double) CalcDeviationD((float)alt, (float)glat, (float)glon, (float)time));
}

static float CalcDeviationD(float alt, float glat, float glon, float time)
{
	int n,m,D3,D4;
	float tc[13][13],dp[13][13],
			sp[13],cp[13],pp[13],dtr,a,b,re,
			a2,b2,c2,a4,b4,c4,otime,oalt,
			olat,olon,dt,rlon,rlat,srlon,srlat,crlon,crlat,srlat2,
			crlat2,q,q1,q2,ct=0.0,st=0.0,r2,r=0.0,d,ca=0.0,sa=0.0,aor,ar,br,bt,bpp,
			par,temp1,temp2,parp,bx, epoch;
	float by, bp;
	float *p = snorm;
	float dec=0.;

	/* INITIALIZE CONSTANTS */
	sp[0] = 0.;
	cp[0] = *p = pp[0] = 1.;
	dp[0][0] = 0.;
	a  = 6378.137;
	b  = 6356.7523142;
	re = 6371.2;
	a2 = a*a;
	b2 = b*b;
	c2 = a2-b2;
	a4 = a2*a2;
	b4 = b2*b2;
	c4 = a4 - b4;

	epoch = 2005.;

	otime = oalt = olat = olon = -1000.;
	dt = time - epoch;
	dtr = M_PI/180.;
	rlon = glon*dtr;
	rlat = glat*dtr;
	srlon = sin(rlon);
	srlat = sin(rlat);
	crlon = cos(rlon);
	crlat = cos(rlat);
	srlat2 = srlat*srlat;
	crlat2 = crlat*crlat;
	sp[1] = srlon;
	cp[1] = crlon;

	/* CONVERT FROM GEODETIC COORDS. TO SPHERICAL COORDS. */
	if(alt != oalt || glat != olat)
	{
		q  = sqrt(a2-c2*srlat2);
		q1 = alt*q;
		q2 = ((q1+a2)/(q1+b2))*((q1+a2)/(q1+b2));
		ct = srlat/sqrt(q2*crlat2+srlat2);
		st = sqrt(1.0-(ct*ct));
		r2 = (alt*alt)+2.0*q1+(a4-c4*srlat2)/(q*q);
		r  = sqrt(r2);
		d  = sqrt(a2*crlat2+b2*srlat2);
		ca = (alt+d)/r;
		sa = c2*crlat*srlat/(r*d);
	}

	if(glon != olon)
	{ 
		for(m=2; m<=MAXDEG; m++)
		{ 
			sp[m] = sp[1]*cp[m-1]+cp[1]*sp[m-1]; 
			cp[m] = cp[1]*cp[m-1]-sp[1]*sp[m-1]; 
		} 
	} 

	aor = re/r; 
	ar = aor*aor; 
	br = bt = bp = bpp = 0.0; 
	for(n=1; n<=MAXDEG; n++)
	{
		ar = ar*aor; 
		for(m=0,D3=1,D4=(n+m+D3)/D3; D4>0; D4--,m+=D3)
		{
			/*
				COMPUTE UNNORMALIZED ASSOCIATED LEGENDRE POLYNOMIALS
				AND DERIVATIVES VIA RECURSION RELATIONS
			*/
			if(alt != oalt || glat != olat)
			{
				if(n == m)
				{
					*(p+n+m*13) = st**(p+n-1+(m-1)*13); 
					dp[m][n] = st*dp[m-1][n-1]+ct**(p+n-1+(m-1)*13); 
				}
				else if(n == 1 && m == 0)
				{
					*(p+n+m*13) = ct**(p+n-1+m*13); 
					dp[m][n] = ct*dp[m][n-1]-st**(p+n-1+m*13); 
				}
				else if(n > 1 && n != m)
				{
					if(m > n-2)
						*(p+n-2+m*13) = 0.0;
					if(m > n-2)
						dp[m][n-2] = 0.0;
					*(p+n+m*13) = ct**(p+n-1+m*13)-k[m][n]**(p+n-2+m*13);
					dp[m][n] = ct*dp[m][n-1] - st**(p+n-1+m*13)-k[m][n]*dp[m][n-2];
				}
			}

			/*
				TIME ADJUST THE GAUSS COEFFICIENTS
			*/
			if(time != otime)
			{
				tc[m][n] = c[m][n]+dt*cd[m][n];
				if(m != 0)
					tc[n][m-1] = c[n][m-1]+dt*cd[n][m-1];
			}
			/*
				ACCUMULATE TERMS OF THE SPHERICAL HARMONIC EXPANSIONS
			*/
			par = ar**(p+n+m*13);
			if(m == 0)
			{
				temp1 = tc[m][n]*cp[m]; 
				temp2 = tc[m][n]*sp[m]; 
			}
			else
			{ 
				temp1 = tc[m][n]*cp[m]+tc[n][m-1]*sp[m]; 
				temp2 = tc[m][n]*sp[m]-tc[n][m-1]*cp[m]; 
			} 
			bt = bt-ar*temp1*dp[m][n]; 
			bp += (fm[m]*temp2*par); 
			br += (fn[n]*temp1*par); 
			/*
				SPECIAL CASE:  NORTH/SOUTH GEOGRAPHIC POLES
			*/
			if(st == 0.0 && m == 1)
			{
				if(n == 1)
					pp[n] = pp[n-1];
				else
					pp[n] = ct*pp[n-1]-k[m][n]*pp[n-2];

				parp = ar*pp[n];
				bpp += (fm[m]*temp2*parp);
			}
		}
	}

	if(st == 0.0)
		bp = bpp; 
	else
		bp /= st; 

	/*
		ROTATE MAGNETIC VECTOR COMPONENTS FROM SPHERICAL TO
		GEODETIC COORDINATES
	*/
	bx = -bt*ca-br*sa;
	by = bp;
	/*
		COMPUTE DECLINATION (DEC), INCLINATION (DIP) AND
		TOTAL INTENSITY (TI)
	*/
	dec = atan2(by,bx)/dtr;
	return(dec);
}

