/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include <DLServer.h>
#include <handera/Vga.h>
#include "prefs.h"
#include "util.h"
#include "resource.h"
#include "forms.h"
#include "def.h"

Boolean StartOdoFormHandler(EventPtr event)
{
	Char buf[16];
	double dist;
	UInt32 l, n, m;
	char *s;
	Boolean handled = false;
	FormPtr	form = FrmGetActiveForm();

	switch(event->eType)
	{
		case frmOpenEvent:
			if(UtilIsVGA())
				VgaFormModify(form, vgaFormModify160To240);

			dist = PrefsGetPrefs()->dist_f;
			PrefsNauticToPref(&dist, NULL);
			n = (UInt32)(dist/1000.);
			StrPrintF(buf, "%05ld", n);
			UtilSetFieldText(form, StartOdoFieldKMID, buf);
			n = dist-(n*1000);
			StrPrintF(buf, "%03ld", n);
			UtilSetFieldText(form, StartOdoFieldMID, buf);

			FrmDrawForm(form);
			handled = true;
			break;
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID)
			{
				case StartOdoOKID:
					s = UtilGetString(form, StartOdoFieldKMID);
					n = StrAToI(s);
					s = UtilGetString(form, StartOdoFieldMID);
					m = StrAToI(s);
					l = StrLen(s);
					if(l < 3)
						m *= 10;
					if(l < 2)
						m *= 10;
					dist = (double)n + (double)m/1000.;
					dist *= 1000.;
					dist = 1./dist;
					PrefsNauticToPref(&dist, NULL);
					dist = 1./dist;
					PrefsGetPrefs()->dist_f = dist;
					FrmUpdateForm(UtilLastFormId(), frmRedrawUpdateCode);
					FrmReturnToForm(0);
					handled = true;
					break;
				case StartOdoCancelID:
					FrmUpdateForm(UtilLastFormId(), frmRedrawUpdateCode);
					FrmReturnToForm(0);
					handled = true;
					break;
				default:
					break;
			}
			break;
		default:
			break;
	}
	return(handled);
}

