/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <PalmOS.h>
#include "main.h"
#include "util.h"
#include "time.h"

#define HOUR2SEC 3600

static Int32 offset = 0;
static DateTimeType datetime, gpstime;
static Char time_string[timeStringLength+4];
static Char date_string[dateStringLength];
static DateFormatType pref_date_format;
static TimeFormatType pref_time_format;

static Char *OFFSET[] =	{
				"+13", "+12", "+11", "+10", "+9", "+8", "+7",
				"+6", "+5", "+4", "+3", "+2", "+1", "+0",
				"-1", "-2", "-3", "-4", "-5", "-6", "-7",
				"-8", "-9", "-10", "-11", "-12", NULL
						};

void TimeInit()
{
	pref_date_format = (DateFormatType)PrefGetPreference(prefDateFormat);
	pref_time_format = (TimeFormatType)PrefGetPreference(prefTimeFormat);

	TimSecondsToDateTime(TimGetSeconds(), &gpstime);
}

void TimeFinish()
{
}

Int16 TimeGetDefOffset()
{
	Int16 i;
	Int16 def_off;

	if(UtilIsOS(4, 0))
	{
		Int16 tz = (Int16)PrefGetPreference(prefTimeZone);
		Int16 ds = (Int16)PrefGetPreference(prefDaylightSavingAdjustment);

		def_off = (Int16)((tz + ds) / 60.);
	}
	else
		def_off = 0;

	i = 0;
	while((OFFSET[i] != NULL) && (StrAToI(OFFSET[i]) != def_off))
		i++;

	return(i);
}

void TimeGetOffsets(Int16 *cnt, Char ***items)
{
	Int16 i = 0;

	while(OFFSET[i] != NULL)
		i++;
	*cnt = i;
	*items = OFFSET;
}

void TimeSetCurrentOffset(Int16 index)
{
	offset = StrAToI(OFFSET[index]);
}

Int32 TimeGetTimeOffset()
{
	return(offset);
}

void TimeSetTime(Int16 hour, Int16 min, Int16 sec)
{
	gpstime.second = sec;
	gpstime.minute = min;
	gpstime.hour   = hour;
}

void TimeSetDate(Int16 year, Int16 month, Int16 day)
{
	// we can do it, since Y2k has passed...
	gpstime.year  = year + 2000;
	gpstime.month = month;
	gpstime.day   = day;
}

void TimeSetOwnDate()
{
	DateTimeType dt;
	TimSecondsToDateTime(TimGetSeconds(), &dt);
	gpstime.year  = dt.year;
	gpstime.month = dt.month;
	gpstime.day   = dt.day;
}

// shows always 24 hour format and never am/pm
Char *TimeGetTimeString()
{
	DateTimeType *datetime;
	char c;

	datetime = TimeGetDateTime();

	switch(pref_time_format)
	{
		case tfComma24h:
				c = ',';
			break;
		case tfDot:
		case tfDotAMPM:
		case tfDot24h:
				c = '.';
			break;
		case tfColon:
		case tfColonAMPM:
		case tfColon24h:
		default:
				c = ':';
			break;
	}

	StrPrintF(time_string, "%02d%c%02d%c%02d",
				datetime->hour, c, datetime->minute, c,
				datetime->second);
	return(time_string);
}

Char *TimeGetDateString()
{
	DateTimeType *datetime;

	datetime = TimeGetDateTime();

	DateToAscii(datetime->month, datetime->day, datetime->year,
				pref_date_format, date_string);
	return(date_string);
}

UInt32 TimeGetGPSSeconds()
{
	return(TimDateTimeToSeconds(&gpstime));
}

DateTimeType *TimeGetDateTime()
{
	datetime = gpstime;

	if(datetime.day != 0)
		TimAdjust(&datetime, offset * HOUR2SEC);

	return(&datetime);
}

void TimeSetPalmTime()
{
	TimSetSeconds(TimDateTimeToSeconds(TimeGetDateTime()));
}

