/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _track_h_
#define _track_h_

#include "nmea.h"
#include "dist.h"
#include "resource.h"
#include "def.h"

typedef struct
{
	char version[2];
	UInt16 category;
	Char name[MAX_TRACK_NAME_LENGTH];
} trackheader_type;

typedef struct
{
	double lat, lon;
	double dist;
	double arc;
	double x, y;
	Int16 alt;
	UInt16 hdop;
	UInt16 vdop;
	UInt16 pdop;
	UInt8 sat_tracked;
	UInt8 fix_quality;
	UInt16 speed;
	UInt32 time;
} trackpoint_type;

typedef struct
{
	double x, y;
	Int16 alt;
	UInt16 speed;
} trackpoint2_type;

typedef struct
{
	DmOpenRef db;
	UInt32 curr, max;
	Boolean is_last;
	trackpoint2_type p;
	trackpoint2_type *cache;
} trackdraw_iter;

Err TrackGetTracks(LocalID **trackids, UInt16 *cnt) CODE2_SECTION;
Err TrackRemoveDB(Char *trackname) CODE2_SECTION;
void TrackSaveInit(Char *trackname) CODE2_SECTION;
void TrackSaveFinish() CODE2_SECTION;
void TrackSavePosition(nmea_type *nmea, Boolean force) CODE2_SECTION;
UInt32 TrackGetSize() CODE2_SECTION;
UInt32 TrackGetCount() CODE2_SECTION;
UInt32 TrackGetCountByName(Char *dbname) CODE2_SECTION;
UInt32 TrackGetCountById(LocalID db_id) CODE2_SECTION;
Err TrackNewDB(Char *trackname) CODE2_SECTION;
Char *TrackGetName() CODE2_SECTION;
void TrackSetCapturing(Boolean b) CODE2_SECTION;
void TrackDoBeep(Boolean b) CODE2_SECTION;
Boolean TrackIsCapturing() CODE2_SECTION;
Int16 TrackGetSaveInterval() CODE2_SECTION;
Boolean TrackGetData(double *dist, double *area) CODE2_SECTION;
Boolean TrackCalc(double *distP, double *areaP) CODE2_SECTION;
void TrackDraw() CODE2_SECTION;
trackdraw_iter *TrackDrawInit() CODE2_SECTION;
Boolean TrackDrawNext(trackdraw_iter *iter) CODE2_SECTION;
void TrackDrawReset(trackdraw_iter *iter) CODE2_SECTION;
void TrackDrawFinish() CODE2_SECTION;
void TrackSendEvent() CODE2_SECTION;
void TrackSetRenameDB(Char *fname) CODE2_SECTION;
Char *TrackGetRenameDB() CODE2_SECTION;
Err TrackRenameTo(Char *name) CODE2_SECTION;
Err TrackSaveAsPDB(Char *trackname) CODE3_SECTION;
Err TrackSaveAsKML(Char *trackname) CODE3_SECTION;

#endif /* _track_h_ */
