/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#define ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#define HAD_ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#endif

#ifndef ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#define ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#define HAD_ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#endif

#include <PalmOS.h>
#include <BtLib.h>
#include <DLServer.h>
#include <unix_stdarg.h>
#include <handera/Vga.h>
#include <Garmin/Garmin.h>
#include <sony/SonyCLIE.h>
#include <HardwareUtils68K.h>
#include "main.h"
#include "forms.h"
#include "util.h"
#include "help.h"
#include "prefs.h"
#include "sincos.h"
#include "time.h"
#include "soarGEOMAG.h"
#include "resource.h"
//#include "pdouble.h"
#include "def.h"

#ifdef HAD_ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#undef HAD_ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#undef ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#endif

#ifdef HAD_ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#undef HAD_ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#undef ALLOW_ACCESS_TO_INTERNALS_OF_FIELDS
#endif

#define MISSING_RES "*resource missing*"

static const UInt32 MAX_UINT32 = 0L - 1L;
static UInt32 is_os;
static UInt32 mem_used, mem_max;
static Boolean is_vga;
static UInt32 is_sony;
static Boolean is_hires;
static Boolean has_color;
static Boolean has_dia;
static UInt32 has_bt;
static Boolean is_ique;
static Boolean is_styletap;
static UInt16 silk_ref;
static UInt16 hwulib_ref;
static UInt8  hwulib_brght;
static Boolean hwulib_loaded;
static Boolean first_log;
static Char appname[dmDBNameLength];

MemPtr UtilMallocInternal(UInt32 size, Boolean be_silent);

#define FIFO_STEP 5
static UInt16 *fifo;
static UInt16 fifo_size;
static UInt16 fifo_cur;

#define DEVDIFF 30

void UtilExit()
{
	EventType event;
	event.eType = appStopEvent;
	EvtAddEventToQueue(&event);
}

void UtilInit()
{
	UInt32 version;
	UInt16 cardNo;
	LocalID dbID;
	Err err = errNone;

	fifo		= NULL;
	fifo_size	= 0;
	fifo_cur	= 0;
	has_color   = false;
	hwulib_loaded = false;

	mem_used = mem_max = 0L;
	first_log = true;

	if(SysCurAppDatabase(&cardNo, &dbID) != errNone)
		UtilError(DbGETDB);

	if(FtrGet(sysFtrCreator, sysFtrNumROMVersion, &is_os) != errNone)
		is_os = 0;

	if(FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == errNone)
		is_vga = true;
	else
		is_vga = false;

	is_hires = false;
	if(UtilIsOS(3, 0))
	{
		if(FtrGet(sysFtrCreator, sysFtrNumWinVersion, &version) != ftrErrNoSuchFeature)
		{
			if(version >= 4)
			{
				UInt32 attr;

				WinScreenGetAttribute(winScreenDensity, &attr);
				if(attr == kDensityDouble)
					is_hires = true;
			}
		}
	}

	if(UtilIsOS(3, 5))
	{
		err = WinScreenMode(winScreenModeGet, NULL, NULL, NULL, &has_color);
		if(err)
			has_color = false;
	}

	is_ique = false;
	err = FtrGet(garminFtrCreator, garminFtrNumIntegratedGPS, &version);
	if(err != ftrErrNoSuchFeature)
		is_ique = true;

	is_styletap = false;
	err = FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &version);
	if(err == errNone && version == 'stap')
		is_styletap = true;

	has_dia = false;
	err = FtrGet(pinCreator, pinFtrAPIVersion, &version);
	if(err == 0 && version)
	{
		if(SysNotifyRegister(cardNo, dbID,
						sysNotifyDisplayResizedEvent, NULL,
						sysNotifyNormalPriority, NULL) == errNone)
		{
			has_dia = true;
		}
	}

	is_sony = 0;
	silk_ref = sysInvalidRefNum;
	if((err = SysLibFind(sonySysLibNameSilk, &silk_ref)))
		if(err == sysErrLibNotFound)
			err = SysLibLoad('libr', sonySysFileCSilkLib, &silk_ref);

	if(err == errNone)
	{
		err = FtrGet(sonySysFtrCreator, sonySysFtrNumVskVersion, &is_sony);
		if(err != errNone)
		{
			is_sony = 1;
			if(SilkLibOpen(silk_ref) == errNone)
				SilkLibEnableResize(silk_ref);
		}
		else
		{
			if(VskOpen(silk_ref) == errNone)
				VskSetState(silk_ref, vskStateEnable, 1);
		}

		if(SysNotifyRegister(cardNo, dbID,
						sysNotifyDisplayChangeEvent, NULL,
						sysNotifyNormalPriority, NULL) != errNone)
		{
			SilkLibClose(silk_ref);
			silk_ref = sysInvalidRefNum;
		}
	}

	if(FtrGet(btLibFeatureCreator, btLibFeatureVersion, &has_bt) != errNone)
	{
		has_bt = 0;
	}

	hwulib_ref = sysInvalidRefNum;
	err = SysLibFind(kHWUtilsName, &hwulib_ref);
	if(err != errNone)
	{
		err = SysLibLoad(sysFileTLibrary, kHWUtilsCreator, &hwulib_ref);
		if(err == errNone)
			hwulib_loaded = true;
	}

	if(err == errNone)
	{
		if(HWUOpen(hwulib_ref) != errNone)
		{
			hwulib_ref = sysInvalidRefNum;
		}
	}
	else
		hwulib_ref = sysInvalidRefNum;
}

void UtilFinish()
{
	UInt16 cardNo;
	LocalID dbID;

	if(SysCurAppDatabase(&cardNo, &dbID) != errNone)
		UtilError(DbGETDB);

	if(has_dia)
	{
		SysNotifyUnregister(cardNo, dbID, sysNotifyDisplayResizedEvent, sysNotifyNormalPriority);
	}

	if(silk_ref != sysInvalidRefNum)
	{

		SilkLibClose(silk_ref);
		SysNotifyUnregister(cardNo, dbID, sysNotifyDisplayChangeEvent, sysNotifyNormalPriority);
	}

	if(hwulib_ref != sysInvalidRefNum)
	{
		HWUClose(hwulib_ref);
		if(hwulib_loaded)
			SysLibRemove(hwulib_ref);
	}

	if(fifo != NULL)
	{
		UtilFree(fifo);
		fifo_size = 0;
		fifo_cur  = 0;
	}
}

MemPtr UtilMalloc(UInt32 size)
{
	return(UtilMallocInternal(size, false));
}

MemPtr UtilMallocSilent(UInt32 size)
{
	return(UtilMallocInternal(size, true));
}

MemPtr UtilMallocInternal(UInt32 size, Boolean be_silent)
{
	MemPtr p = MemPtrNew(size);
	if(p == 0)
	{
//TRACE("mem starved: %ld/%ld/%ld", size, mem_used, mem_max);
		if(!be_silent)
			FrmCustomAlert(AlertForm, "No Mem!", " ", " ");
	}
	else
	{
		mem_used += size;
		mem_max = MAX(mem_max, mem_used);
//TRACE("mem alloced: %ld/%ld/%ld", size, mem_used, mem_max);
	}
	return(p);
}

void UtilFree(MemPtr p)
{
	UInt32 size = MemPtrSize(p);
	MemPtrFree(p);
	mem_used -= size;
//TRACE("mem freed: %ld/%ld/%ld", size, mem_used, mem_max);
}

void UtilFreeHeap(UInt32* free, UInt32* max)
{
	MemHeapFreeBytes(0, free, max);
}

Boolean UtilHasColor()
{
	return(has_color);
}

UInt32 UtilHasBT()
{
	return(has_bt);
}

Boolean UtilIsOS(UInt8 major, UInt8 minor)
{
	return((is_os >= sysMakeROMVersion(major, minor, 0, sysROMStageDevelopment, 0)));
}

Boolean UtilIsVGA()
{
	return(is_vga);
}

Boolean UtilIsSony()
{
	return(is_sony > 0);
}

Boolean UtilIsHires()
{
	return(is_hires);
}

Boolean UtilIsIque()
{
	return(is_ique);
}

Boolean UtilIsStyleTap()
{
	return(is_styletap);
}

Boolean UtilHasDIA()
{
	return(has_dia);
}

#define UtilCos(x) UtilSin((x)+90)
Int32 UtilSin(Int16 angle)
{
	angle = angle % 360;

	if(angle <= 90)
		return(SIN[angle]);
	if(angle <= 180)
		return(SIN[180 - angle]);
	if(angle <= 270)
		return(-SIN[angle - 180]);
	return(-SIN[360 - angle]);
}

MemHandle UtilGetResource(DmResType type, DmResID resID)
{
	UInt16 card_no;
	LocalID db_id;

	if(SysCurAppDatabase(&card_no, &db_id) == errNone)
	{
		DmOpenRef db;

		db = DmOpenDatabase(card_no, db_id, dmModeReadOnly);
		if(db != NULL)
		{
			UInt16 res_id;
			MemHandle mh = NULL;

			res_id = DmFindResource(db, type, resID, NULL);
			if(res_id >= 0)
				mh = DmGetResourceIndex(db, res_id);

			DmCloseDatabase(db);
			return(mh);
		}
	}
	return(NULL);
}

MemHandle UtilGetResourceStringHandle(UInt16 id)
{
	MemHandle handle;

	handle = UtilGetResource(strRsc, id);
	if(handle == NULL)
		UtilError(HelpMISSINGRES);
	return(handle);
}

void UtilReleaseResourceStringHandle(MemHandle handle)
{
	DmReleaseResource(handle);
}

Char *UtilGetResourceString(UInt16 id)
{
	MemHandle handle;
	Char *msg = NULL;

	handle = UtilGetResourceStringHandle(id);
	if(handle != NULL)
	{
		Char *str;

		str = MemHandleLock(handle);
		msg = UtilStrDup(str);
		MemHandleUnlock(handle);
		UtilReleaseResourceStringHandle(handle);
	}
	
	return(msg);
}

void UtilAlert(UInt16 errMsgId)
{
	Char *str;

	SndPlaySystemSound(sndError);
	str = UtilGetResourceString(errMsgId);
	FrmCustomAlert(AlertForm, str, " ", " ");
	UtilFree(str); 
}

void UtilError(UInt16 errMsgId)
{
	SndPlaySystemSound(sndError);
	HelpSetErrorID(errMsgId);
	UtilPopupForm(HelpForm);
}

void UtilErrorMsg(Char *msg)
{
	SndPlaySystemSound(sndError);
	HelpSetErrorMsg(msg);
	UtilPopupForm(HelpForm);
}

UInt16 UtilGetHelpRes()
{
	LocalID db_id;
	UInt16 card_no, hres;
	MemHandle shandle;

	hres = 0;
	if(SysCurAppDatabase(&card_no, &db_id) == errNone)
	{
		UInt16 form_id;

		form_id = FrmGetActiveFormID();
		shandle = UtilGetResource(formRscType, form_id);
		if(shandle != NULL)
		{
			UInt16 *ptr;

			ptr = (UInt16*)MemHandleLock(shandle);
			hres = *(ptr + 29);
			MemHandleUnlock(shandle);
			DmReleaseResource(shandle);
		}
	}
	return(hres);
}

void UtilFrmHelp(UInt16 helpMsgId)
{
	HelpSetHelpID(helpMsgId);
	UtilPopupForm(HelpForm);
}

void* UtilGetObjectPtr(FormPtr form, UInt16 objectID)
{
	UInt16 oi;

	if(form == NULL)
		form = FrmGetActiveForm();

	oi = FrmGetObjectIndex(form, objectID);
	if(oi != frmInvalidObjectId)
		return(FrmGetObjectPtr(form, FrmGetObjectIndex(form, objectID)));

	return(NULL);
}

void UtilBeep(Int32 freq)
{
	SndCommandType sndCmd;

	sndCmd.cmd = sndCmdFrqOn;
	sndCmd.param1 = freq; // freq in Hz
	sndCmd.param2 = 50; // duration in ms
	//sndCmd.param3 = sndMaxAmp; // amplitude
	sndCmd.param3 = PrefGetPreference(prefAlarmSoundVolume);

	SndDoCmd(NULL, &sndCmd, true);
}

Char *UtilStrDup(const Char *s)
{
	Char *t = UtilMalloc(StrLen(s)+1);
	if(t == 0)
		return(NULL);

	StrCopy(t, s);
	return(t);
}

Boolean UtilGetInt(FormPtr form, UInt16 objectID, Int32 *val, Int16 *len)
{
	Char *text;

	text = UtilGetString(form, objectID);
	if(text != NULL)
	{
		if(val != NULL)
			*val = StrAToI(text);
		if(len != NULL)
			*len = StrLen(text);
		return(true);
	}

	if(val != NULL)
		*val = 0;
	if(len != NULL)
		*len = 0;

	return(false);
}

Char *UtilGetString(FormPtr form, UInt16 objectID)
{
	Char *text;
	FieldPtr fp;

	fp = UtilGetObjectPtr(form, objectID);
	text = FldGetTextPtr(fp);
	return(text);
}

void UtilAppendLog(const Char *text)
{
	DlkSetLogEntry(text, StrLen(text), !first_log);
	DlkSetLogEntry(text, StrLen(text), true);
	first_log = false;
}

void UtilUpdateScrollbar(FieldPtr field, ScrollBarPtr sbar)
{
	UInt16 scrollPos, textHeight, fieldHeight;
	Int16 maxValue;

	if(field == NULL)
		return;
	if(sbar == NULL)
		return;

	FldGetScrollValues(field, &scrollPos, &textHeight, &fieldHeight);
	if(textHeight > fieldHeight)
		maxValue = textHeight - fieldHeight;
	else
		if(scrollPos)
			maxValue = scrollPos;
		else
			maxValue = 0;
	SclSetScrollBar(sbar, scrollPos, 0, maxValue, fieldHeight-1);
}

void UtilScrollField(FieldPtr field, ScrollBarPtr sbar, Int16 linesToScroll)
{
	if(field == NULL)
		return;
	if(sbar == NULL)
		return;

	if(linesToScroll < 0)
	{
		UInt16 blankLines;
		Int16  min, max, value, pageSize;

		blankLines = FldGetNumberOfBlankLines(field);
		FldScrollField(field, -linesToScroll, winUp);

		if(blankLines != 0)
		{
			SclGetScrollBar(sbar, &value, &min, &max, &pageSize);
			if(blankLines > -linesToScroll)
				max += linesToScroll;
			else
				max -= blankLines;

			SclSetScrollBar(sbar, value, min, max, pageSize);
		}
	}
	if(linesToScroll > 0)
	{
		FldScrollField(field, linesToScroll, winDown);
	}
}

void UtilPageScroll(FieldPtr field, ScrollBarPtr sbar, WinDirectionType direction)
{
	if(field == NULL)
		return;
	if(sbar == NULL)
		return;

	if(FldScrollable(field, direction))
	{
		UInt16	linesToScroll;
		Int16	value;
		Int16	min;
		Int16	max;
		Int16	pageSize;

		linesToScroll = FldGetVisibleLines(field) - 1;
		FldScrollField(field, linesToScroll, direction);
		SclGetScrollBar(sbar, &value, &min, &max, &pageSize);

		if(direction == winUp )
			value -= linesToScroll;
		else
			value += linesToScroll;

		SclSetScrollBar(sbar, value, min, max, pageSize);
	}
}

void UtilUpdateString(FormPtr form, UInt16 id, Char *data)
{
	UtilUpdateLabel(form, id, "%s", data);
}

void UtilUpdateChar(FormPtr form, UInt16 id, Char data)
{
	UtilUpdateLabel(form, id, "%c", data);
}

void UtilUpdateDec(FormPtr form, UInt16 id, Int32 a, Int32 b)
{
	UtilUpdateLabel(form, id, "%ld.%ld", a, b);
}

void UtilUpdateSingle(FormPtr form, UInt16 id, Int32 a, Char *s)
{
	UtilUpdateLabel(form, id, "%ld %s", a, s);
}

void UtilUpdateDouble(FormPtr form, UInt16 id, double a, UInt8 dec)
{
	UtilUpdateDoubleUnit(form, id, a, dec, NULL);
}

void UtilUpdateDoubleUnit(FormPtr form, UInt16 id, double a, UInt8 dec, Char *s)
{
	Char fbufs[32];
	Int32 v, d;
	UInt8 i;
	Boolean neg;

	if(s == NULL)
		StrPrintF(fbufs, "%%s%%ld.%%0%hulu", dec);
	else
		StrPrintF(fbufs, "%%s%%ld.%%0%hulu %%s", dec);

	neg = a < 0.;
	a = ABS(a);
	v = (Int32)a;
	a -= (double)v;
	for(i = 0; i < dec; i++)
		a *= 10.;
	d = (Int32)a;

	if(s == NULL)
		UtilUpdateLabel(form, id, fbufs, (neg ? "-" : ""), v, d);
	else
		UtilUpdateLabel(form, id, fbufs, (neg ? "-" : ""), v, d, s);
}

void UtilUpdateLabel(FormPtr form, UInt16 id, Char *format_str, ...)
{
	Char ct[MAX_LABEL_LENGTH];
	UInt16 index;
	va_list args;

	index = FrmGetObjectIndex(form, id);
	FrmHideObject(form, index);

	va_start(args, format_str);
	StrVPrintF(ct, format_str, args);
	va_end(args);

	FrmCopyLabel(form, id, ct);
	FrmShowObject(form, index);
}

void UtilSetFieldText(FormType *form, UInt16 id, Char *text)
{
	FieldPtr field;
	MemHandle txt_handle, old_txt_handle;

	if(text == NULL)
	{
		txt_handle = NULL;
	}
	else
	{
		txt_handle = MemHandleNew(StrLen(text) + 1);
		StrCopy(MemHandleLock(txt_handle), text);
		MemHandleUnlock(txt_handle);
	}

	field = UtilGetObjectPtr(form, id);
	old_txt_handle = FldGetTextHandle(field);
	FldSetTextHandle(field, txt_handle);
	FldDrawField(field);

	if(old_txt_handle)
		MemHandleFree(old_txt_handle);
}

void UtilSelectField(FormType *form, UInt16 objID)
{
	FieldPtr field;
	UInt16 length;
	UInt16 objindex;

	field = UtilGetObjectPtr(form, objID);
	length = FldGetTextLength(field);
	FldSetSelection(field, 0, length);

	objindex = FrmGetObjectIndex(form, objID);
	FrmSetFocus(form, objindex);
}

void UtilSetControlValue(FormPtr form, UInt16 objID, Int16 value)
{
	ControlType *ctrl;

	ctrl = UtilGetObjectPtr(form, objID);
	if(ctrl != NULL)
		CtlSetValue(ctrl, value);
}

Int16 UtilGetControlValue(FormPtr form, UInt16 objID)
{
	ControlType *ctrl;

	ctrl = UtilGetObjectPtr(form, objID);
	return(CtlGetValue(ctrl));
}

void UtilSetEnabled(FormPtr form, UInt16 objID, Boolean b)
{
	ControlType *ctrl;

	ctrl = UtilGetObjectPtr(form, objID);
	CtlSetEnabled(ctrl, b);
}

void UtilSetList(FormPtr form, UInt16 list_id, UInt16 trigger_id,
					Char **items, Int16 cnt, Int16 index)
{
	ListType *list;
	RectangleType rect;

	list = UtilGetObjectPtr(form, list_id);
	LstSetListChoices(list, items, cnt);
	LstSetHeight(list, cnt);
	UtilSetListSelection(form, list_id, trigger_id, index);
	FrmGetObjectBounds(form,
				FrmGetObjectIndex(form, list_id), &rect);
	rect.extent.x = UtilGetMaxStringLength(items, cnt) + 10;
	FrmSetObjectBounds(form,
				FrmGetObjectIndex(form, list_id), &rect);
}

void UtilSetListSelection(FormType *form, UInt16 list_id,
							UInt16 trigger_id, Int16 index)
{
	ListType *list;

	list = UtilGetObjectPtr(form, list_id);
	LstSetSelection(list, index);
	CtlSetLabel(UtilGetObjectPtr(form, trigger_id),
		LstGetSelectionText(list, index));//???
}

Char *UtilGetListSelection(FormType *form, UInt16 list_id)
{
	ListType *list;
	Int16 index;

	list = UtilGetObjectPtr(form, list_id);
	index = LstGetSelection(list);
	if(index != noListSelection)
		return(LstGetSelectionText(list, index));

	return(NULL);
}

Int16 UtilGetObjectHeight(FormPtr form, UInt16 id)
{
	RectangleType r;
	UInt16 index;

	index = FrmGetObjectIndex(form, id);
	FrmGetObjectBounds(form, index, &r);
	return(r.extent.y);
}

void UtilMoveObject(FormPtr form, UInt16 id,
					Int16 y_diff, Boolean draw)
{
	RectangleType r;
	UInt16 index;

	index = FrmGetObjectIndex(form, id);
	FrmGetObjectBounds(form, index, &r);
	if(draw)
	{
		RctInsetRectangle(&r, -2);
		WinEraseRectangle(&r, 0);
		RctInsetRectangle(&r, 2);
	}
	r.topLeft.y += y_diff;
	FrmSetObjectBounds(form, index, &r);
}

void UtilResizeObject(FormPtr form, UInt16 id,
						Int16 y_diff, Boolean draw)
{
    RectangleType r;
	UInt16 index;
	Boolean is_list;
	ListType *list = NULL;
	Int16 list_curr = -1;

	if(y_diff == 0)
		return;

	index = FrmGetObjectIndex(form, id);

	is_list = (FrmGetObjectType(form, index) == frmListObj);

	if(is_list)
	{
		list = UtilGetObjectPtr(form, id);
		list_curr = LstGetSelection(list);
		LstMakeItemVisible(list, 0);
	}

	FrmGetObjectBounds(form, index, &r);
	if(draw)
		FrmHideObject(form, index);
	r.extent.y += y_diff;
	FrmSetObjectBounds(form, index, &r);
	if(draw)
		FrmShowObject(form, index);

	if(is_list)
		LstSetSelection(list, list_curr);
}

void UtilShowObject(FormType *form, UInt16 id, Boolean b)
{
	UInt16 oindex;

	oindex = FrmGetObjectIndex(form, id);

	if(b)
		FrmShowObject(form, oindex);
	else
		FrmHideObject(form, oindex);

	if(UtilIsOS(3, 5) == false)
	{
		// work around a bug in OS3.3 and probably less
		if(FrmGetObjectType(form, oindex) == frmScrollBarObj)
		{
			FieldPtr field = FrmGetObjectPtr(form, oindex);
			field->attr.usable = b;
		}
	}
}

Int16 UtilGetMaxStringLength(Char **items, Int16 cnt)
{
	Int16 i, w;

	w = 0;
	for(i = 0; i < cnt; i++)
		w = MAX(w, FntCharsWidth(items[i], StrLen(items[i])));
	return(w);
}

void UtilDBRemove(const Char *name, Err *err)
{
	LocalID db_id;

	db_id = DmFindDatabase(0, name);
	if(db_id != 0)
	{
		*err = DmDeleteDatabase(0, db_id);
		if(*err != errNone)
			UtilError(DbDELETEFAILED);
	}
}

UInt32 UtilDBGetCountByName(const Char *name)
{
	LocalID db_id;
	UInt32 numRecords = 0;

	db_id  = DmFindDatabase(0, name);
	if(db_id)
		DmDatabaseSize(0, db_id, &numRecords, NULL, NULL);
	return(numRecords);
}

UInt32 UtilDBGetCountById(const LocalID db_id)
{
	UInt32 numRecords = 0;
	DmDatabaseSize(0, db_id, &numRecords, NULL, NULL);
	return(numRecords);
}

UInt16 UtilDBGetCountInCategory(const Char *name, UInt16 category)
{
	LocalID db_id;
	UInt16 numRecords = 0;

	db_id  = DmFindDatabase(0, name);
	if(db_id)
	{
		DmOpenRef db = DmOpenDatabase(0, db_id, dmModeReadOnly);
		if(db != 0)
		{
			numRecords = DmNumRecordsInCategory(db, category);
			DmCloseDatabase(db);
		}
	}
	return(numRecords);
}

Err UtilGetDBList(UInt32 creator, UInt32 type, LocalID **ids, UInt16 *cnt)
{
	UInt16 numdb, index;

	*cnt = 0;
	*ids = NULL;
	numdb = DmNumDatabases(0);

	for(index = 0; index < numdb; index++)
	{
		Err err;
		LocalID lid;
		UInt32 dbtype, dbcreator;

		lid = DmGetDatabase(0, index);

		if(lid == 0)
			continue;

		err = DmDatabaseInfo(0, lid, NULL, NULL, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL, &dbtype, &dbcreator);

		if((err == errNone) && (dbtype == type) && (creator == dbcreator))
		{
			(*cnt)++;
		}
	}

	if(*cnt > 0)
	{
		UInt16 i = 0;

		(*ids) = (LocalID*)UtilMalloc((*cnt)*sizeof(LocalID));

		for(index = 0; index < numdb; index++)
		{
			Err err;
			LocalID lid;
			UInt32 dbtype, dbcreator;

			lid = DmGetDatabase(0, index);

			if(lid == 0)
				continue;

			err = DmDatabaseInfo(0, lid, NULL, NULL, NULL, NULL, NULL, NULL,
									NULL, NULL, NULL, &dbtype, &dbcreator);

			if((err == errNone) && (dbtype == type) && (creator == dbcreator))
			{
				(*ids)[i++] = lid;
			}
		}
	}

	return(errNone);
}

Boolean UtilDBExists(const Char *name)
{
	LocalID db_id = DmFindDatabase(0, name);
	if(db_id)
		return(true);

	return(false);
}

UInt32 UtilDBGetSize(const Char *name)
{
	LocalID db_id;
	UInt32 totalBytes = 0;

	db_id  = DmFindDatabase(0, name);
	if(db_id)
		DmDatabaseSize(0, db_id, NULL, &totalBytes, NULL);
	return(totalBytes);
}

Err UtilDBGetName(LocalID id, Char *name)
{
	return(DmDatabaseInfo(0, id, name, NULL, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL, NULL, NULL));
}

DmOpenRef UtilDBOpen(const Char *name, UInt32 creator, UInt32 type, UInt16 mode)
{
	DmOpenRef db;
	LocalID db_id;

	db_id  = DmFindDatabase(0, name);
	if(db_id == 0)
	{
		if(DmCreateDatabase(0, name, creator, type, false) != errNone)
		{
			UtilError(DbCREATEFAILED);
			return(NULL);
		}
		db_id  = DmFindDatabase(0, name);
	}

	db = DmOpenDatabase(0, db_id, mode);
	if(db == 0)
	{
		UtilError(DbOPENFAILED);
		return(NULL);
	}
	return(db);
}

Err UtilDBRename(const Char *from, const Char *to)
{
	Err err = errNone;
	LocalID db_id = DmFindDatabase(0, from);
	if(db_id)
	{
		err = DmSetDatabaseInfo(0, db_id, to, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL, NULL, NULL, NULL);

	}
	else
		err = DmGetLastErr();

	return(err);
}

UInt16 UtilDBSaveRecord(const Char *name, UInt32 creator, UInt32 type,
						const void *src, UInt32 bytes, Err *err)
{
	DmOpenRef db;
	Int16 index;

	*err = errNone;
	db = UtilDBOpen(name, creator, type, dmModeWrite);
	if(db == NULL)
	{
		*err = DmGetLastErr();
		return(0);
	}

	index = UtilDBAddRecord(db, src, bytes, err);

	DmCloseDatabase(db);
	UtilDBSetUpdateFlag(name, err);
	return(index);
}

void UtilDBUpdateRecord(const Char *name, UInt32 creator, UInt32 type,
						UInt16 index, const void *src, UInt32 bytes, Err *err)
{
	DmOpenRef db;
	MemHandle mrec;

	*err = errNone;
	db = UtilDBOpen(name, creator, type, dmModeWrite);
	if(db == NULL)
	{
		*err = DmGetLastErr();
		return;
	}

	mrec = DmGetRecord(db, index);
	if(mrec != NULL)
	{
		MemPtr record;

		record = MemHandleLock(mrec);
		DmWrite(record, 0, src, bytes);
		MemHandleUnlock(mrec);
	}
	else
		*err = DmGetLastErr();

	DmReleaseRecord(db, index, true);
	DmCloseDatabase(db);
}

UInt16 UtilDBAddRecord(DmOpenRef db, const void *src, UInt32 bytes, Err *err)
{
	MemHandle record_h;
	MemPtr record;
	UInt16 rec_index;

	*err = errNone;
	rec_index = dmMaxRecordIndex;
	record_h = DmNewRecord(db, &rec_index, bytes);
	if(record_h == 0)
	{
		*err = DmGetLastErr();
		UtilError(DbRECCREATEFAILED);
		return(0);
	}
	record = MemHandleLock(record_h);
	DmWrite(record, 0, src, bytes);
	MemHandleUnlock(record_h);
	DmReleaseRecord(db, rec_index, true);
	return(rec_index);
}

void UtilDBRemRecord(const Char *name, UInt16 index, Err *err)
{
	LocalID db_id;

	*err = errNone;
	db_id = DmFindDatabase(0, name);
	if(db_id != 0)
	{
		DmOpenRef db;

		db = DmOpenDatabase(0, db_id, dmModeWrite);
		if(db != 0)
		{
			DmRemoveRecord(db, index);
			DmCloseDatabase(db);
			UtilDBSetUpdateFlag(name, err);
		}
		else
			*err = DmGetLastErr();
	}
}

void UtilDBSetUpdateFlag(const Char *name, Err *err)
{
	LocalID db_id;

	*err = errNone;
	db_id = DmFindDatabase(0, name);
	if(db_id != 0)
	{
		UInt16 attr;
		DmDatabaseInfo(0, db_id, NULL, &attr, NULL, NULL, NULL,
						NULL, NULL, NULL, NULL, NULL, NULL);
		attr |= dmHdrAttrBackup;
		DmSetDatabaseInfo(0, db_id, NULL, &attr, NULL, NULL, NULL,
						NULL, NULL, NULL, NULL, NULL, NULL);
	}
	else
		*err = DmGetLastErr();
}

void UtilSwitchDisplay(Boolean b)
{
/*
check sysTrapHwrDisplaySleep, davor sysTrapHwrLCDSleep(<3.5)
und HWUEnableDisplay

For m500 etc. use:
static Boolean HwrBacklight(Boolean set, Boolean newState)
 SYS_TRAP(sysTrapHwrBacklightV33);
set = true: set backlight to newState (true/false)
set = false: newState will return current state of backlight

*/
	if(b == true)
		SysLCDBrightness(true, hwulib_brght);
	else
		hwulib_brght = SysLCDBrightness(true, 0);
/*
		hwulib_brght = 0;

	if(hwulib_ref != sysInvalidRefNum)
	{
		HWUEnableDisplay(hwulib_ref, b);
		HWUBlinkLED(hwulib_ref, !b);
	}
*/
}

// in percent
UInt16 UtilGetBatteryState()
{
	Boolean plugged_in;
	UInt8 percent;

	if(UtilIsOS(3, 0) == true)
	{
		SysBatteryInfo(false, NULL, NULL, NULL, NULL, &plugged_in, &percent);
	}
	else
	{
		UInt16 criticalThreshold, left;
		left = SysBatteryInfoV20(false, NULL, &criticalThreshold, NULL, NULL, &plugged_in);
		percent = 100 - (UInt8)(100. * (double)criticalThreshold / (double)left);
	}

	if(plugged_in == true)
		return(0xFFFF);

	return((UInt16)percent);
}

Int32 UtilGetdec(double val, int cnt)
{
	Int32 n, i;

	if(val < 0)
		val = -val;

	n = (Int32)val;
	val -= (double)n;
	for(i = 0; i < cnt; i++)
		val *= 10;
	return((Int32)val);
}

UInt16 UtilLastFormId()
{
	return(FrmGetFormId(FrmGetFirstForm()));
}

const Char *UtilGetAppName()
{
	LocalID db_id;
	UInt16 card_no;

	StrCopy(appname, "cotoGPS");
	if(SysCurAppDatabase(&card_no, &db_id) == errNone)
	{
		DmDatabaseInfo(card_no, db_id, appname, NULL, NULL,
			NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	}
	return(appname);
}

void UtilPushFifo(UInt16 id)
{
	if(fifo == NULL)
	{
		/* init for FIFO_STEP levels */
		fifo_size	= FIFO_STEP;
		fifo_cur	= 0;
		fifo		= UtilMalloc(fifo_size*sizeof(UInt16));
	}

	if(fifo_cur+1 > fifo_size)
	{
		UInt16 *nfifo;
		UInt16 nfifo_size, i;

		/* enlarge by FIFO_STEP */
		nfifo_size = fifo_size + FIFO_STEP;
		nfifo = UtilMalloc(nfifo_size*sizeof(UInt16));
		for(i = 0; i < fifo_size; i++)
			nfifo[i] = fifo[i];
		UtilFree(fifo);
		fifo = nfifo;
		fifo_size = nfifo_size;
	}

	fifo[fifo_cur++] = id;
}

UInt16 UtilPopFifo()
{
	return(fifo[--fifo_cur]);
}

void UtilPopupForm(UInt16 id)
{
	UtilPushFifo(FrmGetActiveFormID());
	FrmGotoForm(id);
}

void UtilReturnToLastForm()
{
	UInt16 last = UtilPopFifo();
	FrmGotoForm(last);
	//FrmUpdateForm(last, frmRedrawUpdateCode);
}

void UtilGetFrmBounds(FormPtr form, RectangleType *rect)
{
//	if(UtilIsOS(4, 0))
//		WinGetBounds(FrmGetWindowHandle(form), rect);
//	else
		FrmGetFormBounds(form, rect);
}

void UtilGetDpyBounds(RectangleType *rect)
{
//	if(UtilIsOS(4, 0))
//		WinGetBounds(WinGetDisplayWindow(), rect);
//	else
	{
		Coord x, y, w, h;

		x = y = 0;
		WinGetDisplayExtent(&w, &h);
		rect->topLeft.x = x;
		rect->topLeft.y = y;
		rect->extent.x = w;
		rect->extent.y = h;
	}
}

void UtilBmpGetDimensions(const BitmapType *bm, Coord *w, Coord *h)
{
	if(UtilIsOS(4, 0))
		BmpGetDimensions(bm, w, h, NULL);
	else
	{
		*w = bm->width;
		*h = bm->height;
	}
}

double UtilGetDouble(Int32 v, Int32 v_fra, UInt16 v_dec)
{
	UInt16 i;
	double d;

	d = 1;
	for(i = 0; i < v_dec; i++)
		d *= 10;

	return((double)v+(double)v_fra/d);
}

double UtilGetDeviation(double lat, double lon)
{
	DateType dt;
	DateTimeType *dtt;
	double pctyear;
	UInt32 days, cursecs;
	static UInt32 lastcalc = 0;
	static double de = 0.;

	cursecs = TimGetSeconds();
	if((lastcalc != 0) && (cursecs < lastcalc+DEVDIFF))
		return(de);

	lastcalc = cursecs;

	// Find current days since 1904
	//DateSecondsToDate(TimGetSeconds(), &dt);
	dtt = TimeGetDateTime();
	dt.year  = (Int16)dtt->year - 1904;
	dt.month = (Int16)dtt->month;
	dt.day   = (Int16)dtt->day;
	days = DateToDays(dt); // Gives me days since 1904 for current date

	// Find days since 1904 for Jan 1 of the current year
	dt.day = 1;
	dt.month = 1;
	days -= DateToDays(dt); // Gives me days since 1904 for Jan 1

	// Find the percentage of the current year for the current date
	pctyear = (double)(days) / 365.25;
	pctyear = pctyear + (double)dt.year + 1904;

/*
pctyear = 2007.50;
TRACE("YEAR: %s", pdouble(pctyear));
TRACE("MAG  80.0 0.0: %s", pdouble(CalcDeviation(0.0, 80.0, 0.0, pctyear)));
TRACE("MAG  80.0 60.0: %s", pdouble(CalcDeviation(0.0, 80.0, 60.0, pctyear)));
TRACE("MAG  80.0 120.0: %s", pdouble(CalcDeviation(0.0, 80.0, 120.0, pctyear)));
TRACE("MAG  80.0 180.0: %s", pdouble(CalcDeviation(0.0, 80.0, 180.0, pctyear)));
TRACE("MAG  80.0 -120.0: %s", pdouble(CalcDeviation(0.0, 80.0, -120.0, pctyear)));
TRACE("MAG  80.0 -60.0: %s", pdouble(CalcDeviation(0.0, 80.0, -60.0, pctyear)));
TRACE("MAG  40.0 0.0: %s", pdouble(CalcDeviation(0.0, 40.0, 0.0, pctyear)));
TRACE("MAG  40.0 60.0: %s", pdouble(CalcDeviation(0.0, 40.0, 60.0, pctyear)));
TRACE("MAG  40.0 120.0: %s", pdouble(CalcDeviation(0.0, 40.0, 120.0, pctyear)));
TRACE("MAG  40.0 180.0: %s", pdouble(CalcDeviation(0.0, 40.0, 180.0, pctyear)));
TRACE("MAG  40.0 -120.0: %s", pdouble(CalcDeviation(0.0, 40.0, -120.0, pctyear)));
TRACE("MAG  40.0 -60.0: %s", pdouble(CalcDeviation(0.0, 40.0, -60.0, pctyear)));
TRACE("MAG  0.0 0.0: %s", pdouble(CalcDeviation(0.0, 0.0, 0.0, pctyear)));
TRACE("MAG  0.0 60.0: %s", pdouble(CalcDeviation(0.0, 0.0, 60.0, pctyear)));
TRACE("MAG  0.0 120.0: %s", pdouble(CalcDeviation(0.0, 0.0, 120.0, pctyear)));
TRACE("MAG  0.0 180.0: %s", pdouble(CalcDeviation(0.0, 0.0, 180.0, pctyear)));
TRACE("MAG  0.0 -120.0: %s", pdouble(CalcDeviation(0.0, 0.0, -120.0, pctyear)));
TRACE("MAG  0.0 -60.0: %s", pdouble(CalcDeviation(0.0, 0.0, -60.0, pctyear)));
TRACE("MAG  -40.0 0.0: %s", pdouble(CalcDeviation(0.0, -40.0, 0.0, pctyear)));
TRACE("MAG  -40.0 60.0: %s", pdouble(CalcDeviation(0.0, -40.0, 60.0, pctyear)));
TRACE("MAG  -40.0 120.0: %s", pdouble(CalcDeviation(0.0, -40.0, 120.0, pctyear)));
TRACE("MAG  -40.0 180.0: %s", pdouble(CalcDeviation(0.0, -40.0, 180.0, pctyear)));
TRACE("MAG  -40.0 -120.0: %s", pdouble(CalcDeviation(0.0, -40.0, -120.0, pctyear)));
TRACE("MAG  -40.0 -60.0: %s", pdouble(CalcDeviation(0.0, -40.0, -60.0, pctyear)));
TRACE("MAG  -80.0 0.0: %s", pdouble(CalcDeviation(0.0, -80.0, 0.0, pctyear)));
TRACE("MAG  -80.0 60.0: %s", pdouble(CalcDeviation(0.0, -80.0, 60.0, pctyear)));
TRACE("MAG  -80.0 120.0: %s", pdouble(CalcDeviation(0.0, -80.0, 120.0, pctyear)));
TRACE("MAG  -80.0 180.0: %s", pdouble(CalcDeviation(0.0, -80.0, 180.0, pctyear)));
TRACE("MAG  -80.0 -120.0: %s", pdouble(CalcDeviation(0.0, -80.0, -120.0, pctyear)));
TRACE("MAG  -80.0 -60.0: %s", pdouble(CalcDeviation(0.0, -80.0, -60.0, pctyear)));
*/


	// Have to multiply the calculated value by -1.0.  I need East to be negative and West to be positive.
	de = CalcDeviation(0.0, 180.*lat/M_PI, 180.*(-1.*lon)/M_PI, pctyear);
	return(de);
}


/*
void UtilListMaps()
{
	Err err;
	DmSearchStateType state;
	UInt16 card_no, num_db;
	LocalID db_id;

	num_db = 0;
	err = errNone;

	while(err == errNone)
	{
		Err err2;
		Char name[dmDBNameLength+1];

		err = DmGetNextDatabaseByTypeCreator(
						(num_db==0), &state, CGDBMapType, CGCreator,
						false, &card_no, &db_id);

		if(err == errNone)
		{
			err2 = DmDatabaseInfo(card_no, db_id, name,
								NULL, NULL, NULL, NULL, NULL,
								NULL, NULL, NULL, NULL, NULL);
			if(err2 == errNone)
			{
				TRACE("db %d: %s", num_db, name);
			}
		}

		num_db++;
	}
}
*/

