/*
 * Copyright (c) 2010, Thomas Runge (trunge@truwo.de)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * 	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * 	  notice, this list of conditions and the following disclaimer in the
 * 	  documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _util_h_
#define _util_h_

#include <PalmOS.h>
#include <sony/System/SonyChars.h>
#include "nmea.h"
#include "def.h"

#define BEEP_ERROR 200
#define BEEP_OK 2000

#define getdec(x) UtilGetdec((x), 3)

#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#define MAX(a, b) (((a) > (b)) ? (a) : (b))
#define ABS(a) (((a) < 0) ? (-a) : (a))
#define M_PI        3.14159265358979323846
#define M_PI_2      1.57079632679489661923
#define M_PI_4      0.78539816339744830962


#define UtilIsResizeableNoDia() (UtilIsVGA() || UtilIsSony())
#define UtilIsResizeable() (UtilIsResizeableNoDia() || UtilHasDIA())
#define UtilCharIsDigit(c) (((c) >= '0') && ((c) <= '9'))
#define UtilCharIsAlpha(c) ((((c) >= 'a') && ((c) <= 'z')) || (((c) >= 'A') && ((c) <= 'Z')))
#define UtilCharIsUpper(c) (((c) >= 'A') && ((c) <= 'Z'))
#define UtilCharIsLower(c) (((c) >= 'a') && ((c) <= 'z'))
#define UtilCharIsSpace(c) ((c) == ' ' || (c) == '\f' || (c) == '\n' || (c) == '\r' || (c) == '\t' || (c) == '\v')

#define UtilIsBtnDwn(c) ((c) == vchrPageDown || (c) == chrDownArrow || (c) == vchrJogDown)
#define UtilIsBtnUp(c) ((c) == vchrPageUp || (c) == chrUpArrow || (c) == vchrJogUp)
#define UtilIsArrowBtn(event) (UtilIsBtnDwn(event->data.keyDown.chr) || UtilIsBtnUp(event->data.keyDown.chr))
#define UtilIsHardBtn(event) (event->data.keyDown.modifiers & commandKeyMask)
#define UtilBtnDirection(event) (UtilIsBtnDwn(event->data.keyDown.chr) ? vchrPageDown : vchrPageUp)

void UtilExit();
void UtilInit();
void UtilFinish();
MemPtr UtilMalloc(UInt32 size);
MemPtr UtilMallocSilent(UInt32 size);
void UtilFree(MemPtr p);
void UtilFreeHeap(UInt32* free, UInt32* max);
MemHandle UtilGetResource(DmResType type, DmResID resID);
Char *UtilGetResourceString(UInt16 id);
MemHandle UtilGetResourceStringHandle(UInt16 id);
void UtilReleaseResourceStringHandle(MemHandle handle);
void UtilAlert(UInt16 errMsgId);
void UtilError(UInt16 errMsgId);
void UtilErrorMsg(Char *msg);
UInt16 UtilGetHelpRes();
void UtilFrmHelp(UInt16 helpMsgId);
Boolean UtilHasColor();
UInt32 UtilHasBT();
Boolean UtilIsOS(UInt8 major, UInt8 minor);
Boolean UtilIsHires();
Boolean UtilIsIque();
Boolean UtilHasDIA();
Boolean UtilIsVGA();
Boolean UtilIsSony();
Boolean UtilIsStyleTap();
void* UtilGetObjectPtr(FormPtr form, UInt16 objectID);
void UtilBeep(Int32 freq);
Char *UtilStrDup(const Char *s);
Boolean UtilGetInt(FormPtr form, UInt16 objectID, Int32 *val, Int16 *len);
Char *UtilGetString(FormPtr form, UInt16 objectID);
void UtilAppendLog(const Char *text);

#define UtilCos(x) UtilSin((x)+90)
Int32 UtilSin(Int16 angle);

Int16 UtilGetObjectHeight(FormPtr form, UInt16 id);
void UtilMoveObject(FormPtr form, UInt16 id,
					Int16 y_diff, Boolean draw);
void UtilResizeObject(FormPtr form, UInt16 id,
						Int16 y_diff, Boolean draw);

void UtilUpdateScrollbar(FieldPtr field, ScrollBarPtr sbar);
void UtilScrollField(FieldPtr field, ScrollBarPtr sbar, Int16 linesToScroll);
void UtilPageScroll(FieldPtr field, ScrollBarPtr sbar, WinDirectionType direction);

void UtilUpdateLabel(FormPtr form, UInt16 id, Char *format_str, ...);
void UtilUpdateString(FormPtr form, UInt16 id, Char *data);
void UtilUpdateChar(FormPtr form, UInt16 id, Char data);
void UtilUpdateDec(FormPtr form, UInt16 id, Int32 a, Int32 b);
void UtilUpdateSingle(FormPtr form, UInt16 id, Int32 a, Char *s);
void UtilUpdateDouble(FormPtr form, UInt16 id, double a, UInt8 dec);
void UtilUpdateDoubleUnit(FormPtr form, UInt16 id, double a, UInt8 dec, Char *s);

void UtilSetFieldText(FormType *formP, UInt16 objID, Char *text);
void UtilSelectField(FormType *formP, UInt16 objID);
void UtilSetControlValue(FormPtr form, UInt16 objID, Int16 value);
Int16 UtilGetControlValue(FormPtr form, UInt16 objID);
void UtilSetEnabled(FormPtr form, UInt16 objID, Boolean b);
void UtilSetList(FormPtr form, UInt16 list_id, UInt16 trigger_id,
					Char **items, Int16 cnt, Int16 index);
void UtilSetListSelection(FormType *form, UInt16 list_id,
							UInt16 trigger_id, Int16 index);
Char *UtilGetListSelection(FormType *form, UInt16 list_id);

void UtilShowObject(FormType *formP, UInt16 objID, Boolean b);
Int16 UtilGetMaxStringLength(Char **items, Int16 cnt);

/* database related */
UInt32 UtilDBGetCountByName(const Char *name);
UInt32 UtilDBGetCountById(const LocalID db_id);
UInt16 UtilDBGetCountInCategory(const Char *name, UInt16 category);
Err UtilGetDBList(UInt32 creator, UInt32 type, LocalID **trackids, UInt16 *cnt);
Boolean UtilDBExists(const Char *name);
UInt32 UtilDBGetSize(const Char *name);
Err UtilDBGetName(LocalID id, Char *name);
DmOpenRef UtilDBOpen(const Char *name, UInt32 creator, UInt32 type, UInt16 mode);
Err UtilDBRename(const Char *from, const Char *to);
UInt16 UtilDBSaveRecord(const Char *name, UInt32 creator, UInt32 type,
						const void *src, UInt32 bytes, Err *err);
void UtilDBUpdateRecord(const Char *name, UInt32 creator, UInt32 type,
						UInt16 index, const void *src, UInt32 bytes, Err *err);
UInt16 UtilDBAddRecord(DmOpenRef db, const void *src, UInt32 bytes, Err *err);
void UtilDBRemRecord(const Char *name, UInt16 index, Err *err);
void UtilDBSetUpdateFlag(const Char *name, Err *err);
void UtilDBRemove(const Char *name, Err *err);

void UtilSwitchDisplay(Boolean b);
UInt16 UtilGetBatteryState();
Int32 UtilGetdec(double val, int cnt);
UInt16 UtilLastFormId();
const Char *UtilGetAppName();

void UtilPushFifo(UInt16 id);
UInt16 UtilPopFifo();
void UtilPopupForm(UInt16 id);
void UtilReturnToLastForm();

void UtilGetDpyBounds(RectangleType *rect);
void UtilGetFrmBounds(FormPtr form, RectangleType *rect);

void UtilBmpGetDimensions(const BitmapType *bm, Coord *w, Coord *h);
double UtilGetDouble(Int32 v, Int32 v_fra, UInt16 v_dec);
double UtilGetDeviation(double lat, double lon);
//void UtilListMaps();

#endif /* _util_h_ */

